from ..requests.client import RequestsClient, AsyncRequestsClient
import typing
from ..core.request_options import RequestOptions

class AirtopRequests(RequestsClient):
    """
    AirtopRequests client that extends the RequestsClient functionality.
    """

    def wait_for_request_completion(
        self,
        request_id: str,
        *,
        timeout_seconds: int = 300,
        interval_seconds: int = 2,
        request_options: typing.Optional[RequestOptions] = None
    ):
        """
        Waits for the given request to complete.

        Parameters
        ----------
        request_id : str
            The ID of the request to wait for.
        timeout_seconds : int, optional
            Maximum time to wait for completion in seconds
        interval_seconds : int, optional
            Time between status checks in seconds
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RequestStatusResponse
            The final status response of the request

        Raises
        ------
        TimeoutError
            If the request doesn't complete within the timeout period
        ValueError
            If timeout_seconds or interval_seconds are less than or equal to 0
        """
        import time

        if timeout_seconds <= 0:
            raise ValueError("timeout_seconds must be greater than 0")
        if interval_seconds <= 0:
            raise ValueError("interval_seconds must be greater than 0")

        start_time = time.time()
        while True:
            response = self.get_request_status(request_id, request_options=request_options)

            if _is_request_complete(response):
                return response

            if time.time() - start_time > timeout_seconds:
                raise TimeoutError(f"Waiting for request timed out: {request_id}")

            time.sleep(interval_seconds)


class AsyncAirtopRequests(AsyncRequestsClient):
    """
    AsyncAirtopRequests client that extends the AsyncRequestsClient functionality.
    """

    async def wait_for_request_completion(
        self,
        request_id: str,
        *,
        timeout_seconds: int = 300,
        interval_seconds: int = 2,
        request_options: typing.Optional[RequestOptions] = None
    ):
        """
        Waits for the given request to complete.

        Parameters
        ----------
        request_id : str
            The ID of the request to wait for.
        timeout_seconds : int, optional
            Maximum time to wait for completion in seconds
        interval_seconds : int, optional
            Time between status checks in seconds
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RequestStatusResponse
            The final status response of the request

        Raises
        ------
        TimeoutError
            If the request doesn't complete within the timeout period
        ValueError
            If timeout_seconds or interval_seconds are less than or equal to 0
        """
        import asyncio
        import time

        if timeout_seconds <= 0:
            raise ValueError("timeout_seconds must be greater than 0")
        if interval_seconds <= 0:
            raise ValueError("interval_seconds must be greater than 0")

        start_time = time.time()
        while True:
            response = await self.get_request_status(request_id, request_options=request_options)

            if _is_request_complete(response):
                return response

            if time.time() - start_time > timeout_seconds:
                raise TimeoutError(f"Waiting for request timed out: {request_id}")

            await asyncio.sleep(interval_seconds)


def _is_request_complete(response):
    """
    Helper function to check if a request is complete.

    Parameters
    ----------
    response : RequestStatusResponse
        The response from get_request_status

    Returns
    -------
    bool
        True if the request is complete, False otherwise
    """
    return response.status in ["completed", "failed"]


# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing_extensions
import typing
from .monitor_config_include_visual_analysis import MonitorConfigIncludeVisualAnalysis
from ..core.serialization import FieldMetadata
import pydantic
from .interval_monitor_config import IntervalMonitorConfig
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class MonitorConfig(UniversalBaseModel):
    include_visual_analysis: typing_extensions.Annotated[
        typing.Optional[MonitorConfigIncludeVisualAnalysis], FieldMetadata(alias="includeVisualAnalysis")
    ] = pydantic.Field(default=None)
    """
    If set to 'enabled', Airtop AI will also analyze the web page visually when executing the condition check. If set to 'disabled', no visual analysis will be conducted.
    """

    interval: typing.Optional[IntervalMonitorConfig] = pydantic.Field(default=None)
    """
    Configuration for the interval monitor.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
import typing
from ..core.request_options import RequestOptions
from ..types.request_status_response import RequestStatusResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper


class RequestsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_request_status(
        self, request_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RequestStatusResponse:
        """
        Parameters
        ----------
        request_id : str
            The ID of the request to check.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RequestStatusResponse
            OK

        Examples
        --------
        from airtop import Airtop

        client = Airtop(
            api_key="YOUR_API_KEY",
        )
        client.requests.get_request_status(
            request_id="123e4567-e89b-12d3-a456-426614174000",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"requests/{jsonable_encoder(request_id)}/status",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    RequestStatusResponse,
                    parse_obj_as(
                        type_=RequestStatusResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRequestsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_request_status(
        self, request_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RequestStatusResponse:
        """
        Parameters
        ----------
        request_id : str
            The ID of the request to check.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RequestStatusResponse
            OK

        Examples
        --------
        import asyncio

        from airtop import AsyncAirtop

        client = AsyncAirtop(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.requests.get_request_status(
                request_id="123e4567-e89b-12d3-a456-426614174000",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"requests/{jsonable_encoder(request_id)}/status",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    RequestStatusResponse,
                    parse_obj_as(
                        type_=RequestStatusResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ErrorDetail(UniversalBaseModel):
    location: typing.Optional[str] = pydantic.Field(default=None)
    """
    Where the error occurred, e.g. 'body.items[3].tags' or 'path.thing-id'
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error message text
    """

    value: typing.Optional[typing.Optional[typing.Any]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
